@@ -49,6 +49,86 @@
 #include "etc.h"
 
 
+
+
+
+/* -------------------------------------------------------------------
+ * ӥ塼ݡ(ǥХɸ)
+ * 
+ *                     (WX2,WY2)                 (SX2,SY2)
+ *                   
+ *                                         
+ *       (WXp,WYp)                    (SXp,SYp)
+ *                                             
+ *                                             
+ *                   
+ *   (WX1,WY1)                       (SX1,SY1)         
+ *           ɥ                    ӥ塼ݡ
+ * 
+ */
+gint view_port_set(gdouble vx1, gdouble vy1, gdouble vx2, gdouble vy2)
+{
+//#define SET_VIEWPORT
+
+	/* ViewPort Aspect */
+	view_port_aspect = (vx2-vx1) / (vy1-vy2);
+
+	view_port_x1 = vx1;
+	view_port_y1 = vy1;
+	view_port_x2 = vx2;
+	view_port_y2 = vy2;
+
+#ifdef SET_VIEWPORT
+g_print("ViewPort(%f,%f)-(%f,%f)  ViewPortAspect = %f\n", SX1, SY1, SX2, SY2, ViewPortAspect);
+#endif
+
+	return 1;
+}
+
+
+
+/* -------------------------------------------------------------------
+ * ɥ(ɺɸ)
+ * 
+ * ViewPort ꤷƤꤹ
+ * 
+ *                     (WX2,WY2)                 (SX2,SY2)
+ *                   
+ *                                         
+ *       (WXp,WYp)                    (SXp,SYp)
+ *                                             
+ *                                             
+ *                   
+ *   (WX1,WY1)                       (SX1,SY1)         
+ *           ɥ                    ӥ塼ݡ
+ * 
+ */
+gint world_window_set_center(gdouble mag, gdouble mag_x, gdouble mag_y, gdouble view_x)
+{
+	gdouble WXlength, WYlength;
+
+	if (mag == 0) return -1;
+
+	WXlength = view_x / mag;
+	WYlength = WXlength / view_port_aspect;
+
+	world_window_x1 = mag_x - (WXlength / 2);
+	world_window_y1 = mag_y - (WYlength / 2);
+	world_window_x2 = mag_x + (WXlength / 2);
+	world_window_y2 = mag_y + (WYlength / 2);
+
+
+	window_left   =	world_window_x1;
+	window_top	  =	world_window_y2;
+	window_right  =	world_window_x2;
+	window_bottom =	world_window_y1;
+
+
+	return 0;
+}
+
+
+
 /* ---------------------------------------------------------------------
  * 
  * 
@@ -66,6 +146,9 @@ void Draw(GtkWidget *widget)
 	ELLIPSE_LIST *pe;
 
 
+	PenSet(gc, 1, sagcad_color.Back);
+	gdk_draw_rectangle (pixmap, gc, TRUE, 0, 0, widget->allocation.width * 3, widget->allocation.height * 3);
+
 	if (printer.Draw_View == DRAWING_VIEW) WakuWakuDraw(widget);
 
 
@@ -195,28 +278,16 @@ void Draw(GtkWidget *widget)
  */
 int ToDeviceX(double x)
 {
-	int Ret = 0;
-//	double k;
-	double lo, hi;
+	gdouble up, down, ret;
 
-	/* ǥץ쥤 */
-	if (DrawObject == 0) {
-//		lo = sagcad_system.ViewX/Mag;
-//		hi = x - MagX + ((sagcad_system.ViewX / Mag) / 2);
-
-		lo = ToDevice_X_lo;
-		hi = x - MagX + ToDevice_X_hi;
-
-		return (int) (sagcad_system.DeviceX * (hi/lo));
+	up = (x - world_window_x1) * (view_port_x2 - view_port_x1);
+	down = (world_window_x2 - world_window_x1);
+	if (down == 0) {
+		// error
 	}
+	ret = (up / down) + view_port_x1;
 
-	/* ץ󥿡 */
-//	else if (DrawObject == 1) {
-//		k = (sagcad_printer.Horzres / sagcad_printer.Printer_width) * (1440 / sagcad_printer.p_dpiX);
-//		Ret = (int) ( ((x + (sagcad_printer.Printer_width / 2) - MagX) * k) );
-//	}
-
-	return Ret;
+	return (int) ret;
 }
 
 
@@ -235,31 +306,16 @@ int ToDeviceX(double x)
  */
 int ToDeviceY(double y)
 {
-	int Ret = 0;
-//	double k;
-	double lo, hi;
+	gdouble up, down, ret;
 
-
-	/* ǥץ쥤 */
-	if (DrawObject == 0) {
-//		lo = (sagcad_system.ViewX * sagcad_system.Aspect) / Mag;
-//		hi = y - MagY + ( (sagcad_system.ViewX * sagcad_system.Aspect) / (2 * Mag) );
-
-		lo = ToDevice_Y_lo;
-		hi = y - MagY + ToDevice_Y_hi;
-
-		return (int) (sagcad_system.DeviceY * ( 1 - (hi/lo) ));
+	up = (y - world_window_y1) * (view_port_y1 - view_port_y2);
+	down = (world_window_y2 - world_window_y1);
+	if (down == 0) {
+		// error
 	}
+	ret = view_port_y1 - (up / down);
 
-#ifdef TEST
-	/* ץ󥿡 */
-	else if (DrawObject == 1) {
-		k = (sagcad_printer.Vertres / sagcad_printer.Printer_height) * (1440 / sagcad_printer.p_dpiY);
-		Ret =(int) ( ((y + (-sagcad_printer.Printer_height / 2) - MagY) * k) );
-	}
-#endif
-
-	return Ret;
+	return (int) ret;
 }
 
 
@@ -273,11 +329,16 @@ int ToDeviceY(double y)
  */
 double ToViewX(int x)
 {
-	double dumy;
+	gdouble up, down, ret;
 
+	up = (world_window_x2 - world_window_x1) * (x - view_port_x1);
+	down = (view_port_x2 - view_port_x1);
+	if (down == 0) {
+		// error
+	}
+	ret = (up / down) + world_window_x1;
 
-	dumy = (double) (x / ToViewK) - ( sagcad_system.DeviceX / (2 * ToViewK)) + MagX;
-	return dumy;
+	return ret;
 }
 
 
@@ -291,10 +352,16 @@ double ToViewX(int x)
  */
 double ToViewY(int y)
 {
-	double dumy;
+	gdouble up, down, ret;
 
-	dumy = (double) ((-y / ToViewK) + (sagcad_system.DeviceY / (2 * ToViewK)) + MagY);
-	return dumy;
+	up = (world_window_y2 - world_window_y1) * (view_port_y1 - y);
+	down = (view_port_y1 - view_port_y2);
+	if (down == 0) {
+		// error
+	}
+	ret = (up / down) + world_window_y1;
+
+	return ret;
 }
 
 
@@ -1605,6 +1672,12 @@ void SetMag(double MagData)
 	if (MagData < 0.002) MagData = (float) 0.002;
 	if (MagData > 2000) MagData = 2000;
 	Mag = MagData;
+
+	world_window_set_center(Mag, 
+							MagX, 
+							MagY, 
+							sagcad_system.ViewX * 3);
+
 	DC = (double)(sagcad_system.Search / Mag);
 	/* --- < Ψ > --- */
 	sagcad_system.ScrollPitch = (float)(sg((sagcad_system.ViewX/abs(sagcad_system.Splits))/Mag, 2));
@@ -1612,7 +1685,7 @@ void SetMag(double MagData)
 	sprintf(DumyStr, "SP %s", str);
 	set_label_text(scroll_pitch, DumyStr);
 
-	window_set();
+//	window_set();
 
 	/*  */
 	Redraw();
