@@ -705,8 +705,11 @@ int ending(void)
 	g_print("  Adviser         mosoku <f_mosoku@lycos.jp>\n");
 	g_print("                  allentown <pulsar@sea.plala.or.jp>\n");	// Narita Koichi 
 	g_print("                  0110110 <sora@tka.att.ne.jp>\n");		// Yoshihiro Horii 
+	g_print("                  yamaguchi <yamaguchi@tac.tsukuba.ac.jp>\n");
 	g_print("  Japanese Font   Yoshimune Kobayashi <cfg47520@syd.odn.ne.jp>\n");
 	g_print("  German          Thomas Formella <thomas-formella@web.de>\n");
+	g_print("  English         Doug Fortune <pentam@cncKITS.com>\n");
+	g_print("                  Matt Kasdorf <kasdorfm@knightfire.com>\n");
 	g_print("\n");
 
 	return 1;
